#!/bin/bash
# Simple wrapper script to update OWASP ModSecurity core ruleset from Internet.
# Download-only mode, isn't destructive.
# Call periodically e.g. by symlinking from /etc/cron.weekly.
# Part of website-manager-2 package.

GREP=$(which grep)		# grep command call
HEAD=$(which head)		# head command call
SED=$(which sed)		# sed command call
TAR=$(which tar)		# tar command call
WGET=$(which wget)		# wget command call

DEBIAN_MAIOR="$(cat /etc/debian_version | /usr/bin/cut -c1)"
MODSEC_DIR="/etc/apache2/modsecurity/modsecurity-crs"
MODSEC_TIMEOUT=1
MODSEC_URL="https://github.com/SpiderLabs/owasp-modsecurity-crs/tarball/master"
MODSEC_USERAGENT="WSM2 ModSecurity CRS Updater"

MSG_MODSEC_EXISTS="Refused to overwrite existing CRS at"
MSG_MODSEC_NEWCRSFOUND="New ModSecurity OWASP CRS found!"
MSG_MODSEC_NEWCRSREADY="New CRS is ready to use at"
MSG_MODSEC_UNTARERROR="Error extracting"
MSG_MODSEC_WGETERROR="Unable to download CRS from"

# Only root allowed to run.
if [ ! $UID -eq 0 ]; then exit 1; fi

# Minimum Debian version is Wheezy.
if [ $DEBIAN_MAIOR -lt 7 ]; then exit 1; fi

# Including config file (if any).
MODSEC_CONF="/etc/default/wsm2-modsec-update-owasp"
if [ -r "$MODSEC_CONF" ]; then . $MODSEC_CONF; fi

# Getting the last version as a tarball.
MODSEC_TMPFILE="/tmp/modsecurity-crs-owasp-master.tgz"
if [ -f "$MODSEC_TMPFILE" ]; then rm "$MODSEC_TMPFILE"; fi
RESULT=$($WGET "$MODSEC_URL" -O "$MODSEC_TMPFILE" --no-check-certificate -U "$MODSEC_USERAGENT" -T "$MODSEC_TIMEOUT" >/dev/null 2>&1; echo $?)
if [ "$RESULT" -ne 0 ]; then
    echo "$MSG_MODSEC_WGETERROR $MODSEC_URL"
    if [ -f "$MODSEC_TMPFILE" ]; then rm "$MODSEC_TMPFILE"; fi
    exit 1
fi
# Check version.
CRSVERLINE=$($TAR xzf "$MODSEC_TMPFILE" -O --wildcards */CHANGES | $GREP -i 'version' | $HEAD -n1)
CRSVER=$(echo "$CRSVERLINE" | $SED "s/.\+ersion\s\+\([.0-9]\+\).*/\1/") #"
if [ -d "$MODSEC_DIR/$CRSVER" ]; then
    echo "$MSG_MODSEC_EXISTS $MODSEC_DIR/$CRSVER"
    if [ -f "$MODSEC_TMPFILE" ]; then rm "$MODSEC_TMPFILE"; fi
    exit 1
fi

# New CRS found!
echo -e "$MSG_MODSEC_NEWCRSFOUND\n$CRSVERLINE"
mkdir "$MODSEC_DIR/$CRSVER"
RESULT=$($TAR xzf "$MODSEC_TMPFILE" -C "$MODSEC_DIR/$CRSVER" --strip-components=1; echo $?)
if [ "$RESULT" -ne 0 ]; then
    echo "$MSG_MODSEC_UNTARERROR $MODSEC_TMPFILE -> $MODSEC_DIR/$CRSVER"
    if [ -f "$MODSEC_TMPFILE" ]; then rm "$MODSEC_TMPFILE"; fi
    exit 1
fi
# Harden access rights.
chmod -R g-w,o-rwx "$MODSEC_DIR/$CRSVER"
# Cleanup.
if [ -f "$MODSEC_TMPFILE" ]; then rm "$MODSEC_TMPFILE"; fi
# That's all!
echo "$MSG_MODSEC_NEWCRSREADY $MODSEC_DIR/$CRSVER"
